#!/usr/bin/env python
# -*- coding: utf-8 -*-

from gimpfu import *
import unicodedata
import math
import sys,os,os.path,re,glob,traceback
from collections import namedtuple



#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#   The GPL v3 licence is available at: https://www.gnu.org/licenses/gpl-3.0.en.html

# ------------
#| Change Log |
# ------------
# Rel 0.1: Initial release. A little modification Custom Font Layers With Fixed Margins V.0.03 for Gimp 2.10 by Skinnyhouse
#          https://gimplearn.net/viewtopic.php?f=3&t=2614&sid=80366dfd9c7704ff9df895ca83bbcd6a

# Rel 0.2 Code embedded: ofn_tiles_to_layer.py by Ofnuts


################################################ Start ofn_tiles_to_layer ########################################################################################################################
debug=False
def trace(s):
    if debug:
        print s

def createOpts(name,pairs):
    optsclass=namedtuple(name+'Type',[symbol for symbol,label in pairs]+['labels','labelTuples'])
    opts=optsclass(*(
                    range(len(pairs))
                    +[[label for symbol,label in pairs]]
                    +[[(label,i) for i,(symbol,label) in enumerate(pairs)]]
                    ))
    return opts

JoinDefs=createOpts('JoinDefs',[('ROWS_GIVEN','Rows given, compute columns\nPodane wiersze, oblicz kolumny'),('COLUMNS_GIVEN','Columns given, compute rows\nPodane kolumny, oblicz wiersze'),('GUESS','Parse tile names\nAnalizuj nazwy kafelków')])
JoinStackOrder=createOpts('JoinStackOrder',[('FROM_TOP','From top\nZ góry'),('FROM_BOTTOM','From bottom\nZ dołu')])
JoinFlatten=createOpts('JoinFlatten',[('NO','Keep as distinct layers\nZachowaj jako odrębne warstwy'),('YES','Flatten to single layer\nSpłaszcz do pojedynczej warstwy')])
RCOrder=createOpts('RCOrder',[('ROWS_FIRST','Rows first\nNajpierw rzędy'),('COLUMNS_FIRST','Columns first\nNajpierw kolumny')])
Resize=createOpts('Resize',[('NO','No change\nBrak zmiany'),('YES','Fit to result\nDopasuj do wyniku')])
SplitInPlace=createOpts('SplitInPlace',[('IN_PLACE','Keep in place\nPrzytrzymać'),('SUPERIMPOSE','Superimpose\nNakładać')])

def iterateTiles(rows,columns,rowsFirst):
    if rowsFirst:
        for row in range(rows):
            for column in range(columns):
                yield column*rows+row,row*columns+column,row,column
    else:
        for column in range(columns):
            for row in range(rows):
                yield column*rows+row,row*columns+column,row,column
            
def setValues(layerName,rows,columns,width,height,row,column,tileCR,tileRC):
    formatValues={}
    formatValues['layerName']=layerName
    formatValues['rows']=rows
    formatValues['columns']=columns
    formatValues['width']=width
    formatValues['height']=height
    formatValues['count']=rows*columns
    formatValues['row0']=row
    formatValues['row1']=row+1
    formatValues['column0']=column
    formatValues['column1']=column+1
    formatValues['tileCR0']=tileCR
    formatValues['tileCR1']=tileCR+1
    formatValues['tileRC0']=tileRC
    formatValues['tileRC1']=tileRC+1
    return formatValues
        
def layerToTiles(image,layer,namePattern,tileData,inPlace,resize,rcOrder):
    image.undo_group_start()
    
    try:
        if type(layer) is not gimp.Layer:
            raise Exception('Source element must be a plain layer')
        width,height,rows,columns=tileData
        for tileCR,tileRC,row,column in iterateTiles(rows,columns,rcOrder==RCOrder.ROWS_FIRST):
            formatValues=setValues(layer.name,rows,columns,width,height,row,column,tileCR,tileRC)
            try:
                tileName=namePattern.format(**formatValues)
            except KeyError as e:
                raise Exception('No formatting variable called "%s"' % e.args[0])        
            tile=layer.copy()
            image.add_layer(tile,0)
            tile.resize(width,height,-column*width,-row*height)
            tile.name=tileName
            if inPlace==SplitInPlace.SUPERIMPOSE:
                tile.set_offsets(0,0)
            trace('Tile W*H: %d*%d' % (tile.width,tile.height))
        image.remove_layer(layer)
        if inPlace==SplitInPlace.SUPERIMPOSE and resize==Resize.YES:
            image.resize(width,height)

    except Exception as e:
        pdb.gimp_message(e.args[0])
        print traceback.format_exc()
    
    image.undo_group_end()

def tilesGuessedFromNames(layers,rcOrder):
    pattern=re.compile(r'(^(.*\D)?)(\d+)(\D+)(\d+)(\D*)$')
    commonSeps=None
    tilesCoords=[]
    for layer in layers:
        match=pattern.match(layer.name)
        if not match:
            raise Exception('Cannot figure out row or columns in layer "%s"' % layer.name)
        s1,_,n1,s2,n2,s3=match.groups()
        if not commonSeps: # first image
            commonSeps=(s1,s2,s3)
        else:
            if commonSeps!=(s1,s2,s3):
                raise Exception('Layer "%s" doesn\'t match %s**%s**%s' % ((layer.name,)+commonSeps))
        if rcOrder==RCOrder.ROWS_FIRST:
            tc=(int(n2),int(n1),layer)
        else:
            tc=(int(n1),int(n2),layer)
        tilesCoords.append(tc)

    minCol=min([x for x,y,l in tilesCoords])
    maxCol=max([x for x,y,l in tilesCoords])
    minRow=min([y for x,y,l in tilesCoords])
    maxRow=max([y for x,y,l in tilesCoords])
    rows=1+maxRow-minRow
    cols=1+maxCol-minCol
    count=rows*cols
    trace('Guessed: %d rows and %d columns' % (rows,cols))
    if count!=len(tilesCoords):
        raise Exception('Number of layers (%d) does not match the number of expected tiles (%d*%d=%d)' % (len(layers),rows,cols,count))
    tiles=[[None for _ in range(rows)] for _ in range(cols)]
    for x,y,l in tilesCoords:
        tiles[x-minCol][y-minRow]=l
    return tiles,rows,cols

def tilesFromGivenRC(layers,defs,geometry,rcOrder):
    if defs== JoinDefs.ROWS_GIVEN:
        rows,cols=geometry,len(layers)/geometry
    else:
        rows,cols=len(layers)/geometry,geometry
        
    if len(layers)!=rows*cols:
        raise Exception('Number of layers (%d) does not match the number of expected tiles (%d*%d=%d)' % (len(layers),rows,cols,rows*cols))

    if rcOrder==RCOrder.ROWS_FIRST:
        tiles=[layers[x::cols] for x in range(cols)]
    else:
        tiles=[layers[x:x+rows] for x in range(0,len(layers),rows)]
        
    return tiles,rows,cols

def dumpTiles(tiles):
    if not debug:
        return
    trace(tiles)
    for y in range(len(tiles[0])): 
        print ' '.join([l.name.strip() for l in [tiles[x][y] for x in range(len(tiles))]])

def tilesToLayerUnicode(image,defs,geometry,rcOrder,stackOrder,flatten,resize):
    image.undo_group_start()
    try:
        layers=[l for l in image.layers if type(l) is gimp.Layer and l.visible]
        if stackOrder==JoinStackOrder.FROM_BOTTOM:
            layers.reverse()
        if defs==JoinDefs.GUESS:
            tiles,rows,columns=tilesGuessedFromNames(layers,rcOrder)
        else:
            tiles,rows,columns=tilesFromGivenRC(layers,defs,int(geometry),rcOrder)
        
        dumpTiles(tiles)

        tileW,tileH=tiles[0][0].width,tiles[0][0].height
        for y in range(rows):
            for x in range(columns):
                tile=tiles[x][y]
                if (tileW,tileH)!=(tile.width,tile.height):
                        raise Exception('Tile "%s" is not the same size as the previous ones' % tile.name)
                tile.set_offsets(x*tileW,y*tileH)
            
        if resize==Resize.YES:
            image.resize(columns*tileW,rows*tileH)
        if flatten==JoinFlatten.YES: 
            tilesLayer=image.merge_visible_layers(EXPAND_AS_NECESSARY)
            tilesLayer.name='Tiles'
        
    except Exception as e:
        pdb.gimp_message(e.args[0])
        print traceback.format_exc()
    image.undo_group_end()
	
################################################ End ofn_tiles_to_layer ########################################################################################################################
	
def character_groups(image, layer, font_size, symbols, create_new, path, display_keywords, reverse, iter, h_font, h_font_size, h_color):

    '''
    if display_keywords:
        message_text = "Current keywords for grouping unicode characters:\n\nactivity animal arrow asterisk astrology \nballot beauty body braille bubble building bullet \ncandy celebration cereal chess circle clock clothes clubs communication cross "
        message_text += "currency \ndiamonds dice dingbat domino \neating emoticon entertainment \nface fantasy figures finance flag flower food fruit \ngender gestures hands health \nhearts \nmap media medicine misc money music \nnumber \npen people places pointer \nrectangle recycling rhombus "
        message_text += "religion romance runic \nscissors shape sign space spades sport square star stationery sweets suit \ntarot technology tick transport tree triangle \nvegetable vote \nweather."
        old_handler = pdb.gimp_message_get_handler()
        pdb.gimp_message_set_handler(MESSAGE_BOX)
        pdb.gimp_message(message_text)
        pdb.gimp_message_set_handler(old_handler)
    '''
 
    symbol_list = ["dummy"]

    if symbols==0:
        symbols="activity"
    if symbols==1:
        symbols="animal"
    if symbols==2:
        symbols="arrow"
    if symbols==3:
        symbols="asterisk"
    if symbols==4:
        symbols="astrology"
    if symbols==5:
        symbols="ballot"
    if symbols==6:
        symbols="beauty"
    if symbols==7:
        symbols="body"
    if symbols==8:
        symbols="braille"
    if symbols==9:
        symbols="bubble"
    if symbols==10:
        symbols="building"
    if symbols==11:
        symbols="bullet"
    if symbols==12:
        symbols="candy"
    if symbols==13:
        symbols="celebration"
    if symbols==14:
        symbols="cereal"
    if symbols==15:
        symbols="chess"
    if symbols==16:
        symbols="circle"
    if symbols==17:
        symbols="clock"
    if symbols==18:
        symbols="clothes"
    if symbols==19:
        symbols="clubs"
    if symbols==20:
        symbols="communication"
    if symbols==21:
        symbols="cross"
    if symbols==22:
        symbols="currency"
    if symbols==23:
        symbols="diamonds"
    if symbols==24:
        symbols="dice"
    if symbols==25:
        symbols="dingbat"
    if symbols==26:
        symbols="domino"	
    if symbols==27:
        symbols="eating"
    if symbols==28:
        symbols="emoticon"
    if symbols==29:
        symbols="entertainment"
    if symbols==30:
        symbols="face"
    if symbols==31:
        symbols="fantasy"
    if symbols==32:
        symbols="figures"
    if symbols==33:
        symbols="finance"
    if symbols==34:
        symbols="flag"
    if symbols==35:
        symbols="flower"
    if symbols==36:
        symbols="food"
    if symbols==37:
        symbols="fruit"
    if symbols==38:
        symbols="gender"
    if symbols==39:
        symbols="gestures"
    if symbols==40:
        symbols="hands"
    if symbols==41:
        symbols="health"
    if symbols==42:
        symbols="hearts"
    if symbols==43:
        symbols="map"
    if symbols==44:
        symbols="media"
    if symbols==45:
        symbols="medicine"
    if symbols==46:
        symbols="misc"
    if symbols==47:
        symbols="money"
    if symbols==48:
        symbols="music"
    if symbols==49:
        symbols="number"		
    if symbols==50:
        symbols="pen"
    if symbols==51:
        symbols="people"
    if symbols==52:
        symbols="places"
    if symbols==53:
        symbols="pointer"
    if symbols==54:
        symbols="rectangle"
    if symbols==55:
        symbols="recycling"
    if symbols==56:
        symbols="rhombus"
    if symbols==57:
        symbols="religion"
    if symbols==58:
        symbols="romance"
    if symbols==59:
        symbols="runic"
    if symbols==60:
        symbols="scissors"
    if symbols==61:
        symbols="shape"
    if symbols==62:
        symbols="sign"
    if symbols==63:
        symbols="space"
    if symbols==64:
        symbols="spades"
    #if symbols==65:
        #symbols="sport"
    if symbols==65:
        symbols="square"
    if symbols==66:
        symbols="star"
    if symbols==67:
        symbols="stationery"
    if symbols==68:
        symbols="sweets"
    if symbols==69:
        symbols="suit"
    if symbols==70:
        symbols="tarot"
    if symbols==71:
        symbols="technology"
    if symbols==72:
        symbols="tick"
    if symbols==73:
        symbols="transport"
    if symbols==74:
        symbols="tree"
    if symbols==75:
        symbols="triangle"
    if symbols==76:
        symbols="vegetable"
    if symbols==77:
        symbols="vote"
    if symbols==78:
        symbols="weather"
    if symbols==79:
        symbols="Greek_language"
		
    if symbols:

        #### start of unicode character data

        if "anim" in symbols:# animals
            #do_job = True
            #symbol_list = fn_get_unicode_value(128000, 128063,symbol_list)
            symbol_list = fn_get_unicode_value(128049, 128062,symbol_list)
            symbol_list = fn_get_unicode_value(129408, 129431,symbol_list)

        if "arro" in symbols or "arrow plain" in symbols:# arrows
            do_job = True
            symbol_list = fn_get_unicode_value(129024,129095,symbol_list)# from supplementary arrows C
            symbol_list = fn_get_unicode_value(129104,129113,symbol_list)# from supplementary arrows C
            symbol_list = fn_get_unicode_value(129120,129159,symbol_list)# from supplementary arrows C
            symbol_list = fn_get_unicode_value(129168,129179,symbol_list)# from supplementary arrows C

        if "arro" in symbols or "arrow 3d" in symbols:# arrows
            do_job = True
            symbol_list = fn_get_unicode_value(129184,129195,symbol_list)# from supplementary arrows C

        if "arro" in symbols or "arrow curved" in symbols:# arrows
            do_job = True
            symbol_list = fn_get_unicode_value(10552,10561,symbol_list)# from supplementary arrows B
            symbol_list = fn_get_unicode_value(10226,10227,symbol_list)# from supplementary arrows A

        if "aste" in symbols:# asterisks
            do_job = True
            symbol_list = fn_get_unicode_value(10018,10021,symbol_list)
            symbol_list = fn_get_unicode_value(10031,10033,symbol_list)
            symbol_list = fn_get_unicode_value(10040,10043,symbol_list)
            symbol_list = fn_get_unicode_value(10048,0,symbol_list)
            symbol_list = fn_get_unicode_value(10052,10057,symbol_list)
            symbol_list = fn_get_unicode_value(10059,0,symbol_list)

        if "astrol" in symbols or "cons" in symbols:# astrology constellations
            do_job = True
            symbol_list = fn_get_unicode_value(9800,9811,symbol_list)

        if "beaut" in symbols:# beauty
            do_job = True
            symbol_list = fn_get_unicode_value(128132,128136,symbol_list)

        if "body" in symbols:#
            do_job = True
            symbol_list = fn_get_unicode_value(128064,128080,symbol_list)

        if "brai" in symbols:# braille
            do_job = True
            symbol_list = fn_get_unicode_value(10241,10495,symbol_list)

        if "bubb" in symbols or "speech" in symbols:# speech bubbles
            do_job = True
            symbol_list = fn_get_unicode_value(128172,128173,symbol_list)
            symbol_list = fn_get_unicode_value(128488,128498,symbol_list)

        if "buil" in symbols:# buildings
            do_job = True
            symbol_list = fn_get_unicode_value(127967,127984,symbol_list)

        if "bull" in symbols:# bullet symbols
            do_job = True
            symbol_list = fn_get_unicode_value(9152,0,symbol_list)
            symbol_list = fn_get_unicode_value(10070,0,symbol_list)
            symbol_list = fn_get_unicode_value(10084,10087,symbol_list)
            symbol_list = fn_get_unicode_value(10102,10131,symbol_list)

        if "busi" in symbols:# buildings
            do_job = True
            symbol_list = fn_get_unicode_value(128186,128230,symbol_list)

        if "cards spades" in symbols or "spades" in symbols:# playing cards spades
            do_job = True
            symbol_list = fn_get_unicode_value(127137,127150,symbol_list)

        if "cards hearts" in symbols or "hearts" in symbols:# playing cards hearts
            do_job = True
            symbol_list = fn_get_unicode_value(127153,127166,symbol_list)

        if "cards diamonds" in symbols or "diamonds" in symbols:# playing cards diamonds
            do_job = True
            symbol_list = fn_get_unicode_value(127169,127182,symbol_list)

        if "cards clubs" in symbols or "clubs" in symbols:# playing cards clubs
            do_job = True
            symbol_list = fn_get_unicode_value(127185,127198,symbol_list)

        if "card suit" in symbols or "suits" in symbols:# playing cards the four suit symbols
            do_job = True
            symbol_list = fn_get_unicode_value(9824,0,symbol_list)
            symbol_list = fn_get_unicode_value(9827,0,symbol_list)
            symbol_list = fn_get_unicode_value(9829,0,symbol_list)
            symbol_list = fn_get_unicode_value(9830,0,symbol_list)

        if "cele" in symbols: # celebrations
            do_job = True
            symbol_list = fn_get_unicode_value(127872,127893,symbol_list)

        if "cer" in symbols: # cereals
            do_job = True
            symbol_list = fn_get_unicode_value(127805,127806,symbol_list)

        if "chess" in symbols:
            do_job = True
            symbol_list = fn_get_unicode_value(9812,9823,symbol_list)

        if "circ" in symbols:# Geometric Shapes
            do_job = True
            symbol_list = fn_get_unicode_value(9673,0,symbol_list) # Geometric Shapes
            symbol_list = fn_get_unicode_value(9675,9687,symbol_list) # Geometric Shapes
            symbol_list = fn_get_unicode_value(9711,0,symbol_list) # Geometric Shapes
            symbol_list = fn_get_unicode_value(9716,9719,symbol_list) # Geometric Shapes
            symbol_list = fn_get_unicode_value(128900,128907,symbol_list) # Geometric Shapes Extended

        if "clock" in symbols or "time" in symbols:
            do_job = True
            symbol_list = fn_get_unicode_value(9200,9203,symbol_list)
            symbol_list = fn_get_unicode_value(128336,128359,symbol_list)

        if "cloth" in symbols: # clothes
            do_job = True
            symbol_list = fn_get_unicode_value(128081,128098,symbol_list)

        if "comm" in symbols: # communication
            do_job = True
            symbol_list = fn_get_unicode_value(128379,128390,symbol_list)

        if "cross" in symbols:
            do_job = True
            symbol_list = fn_get_unicode_value(10009,10016,symbol_list)

        if "dice" in symbols:
            do_job = True
            symbol_list = fn_get_unicode_value(9856,9861,symbol_list)

        if "dingbat" in symbols or "ding" in symbols:
            do_job = True
            symbol_list = fn_get_unicode_value(9984,10175,symbol_list)

        if "domino" in symbols:
            do_job = True
            symbol_list = fn_get_unicode_value(127024,127123,symbol_list)

        if "ente" in symbols: # entertainment
            do_job = True
            symbol_list = fn_get_unicode_value(127896,127932,symbol_list)

        if "face" in symbols or "emoti" in symbols : # emoticons
            do_job = True
            symbol_list = fn_get_unicode_value(9785,9787,symbol_list)
            symbol_list = fn_get_unicode_value(128512,128591,symbol_list)
            symbol_list = fn_get_unicode_value(129296,129303,symbol_list)
            symbol_list = fn_get_unicode_value(129312,129327,symbol_list)
            symbol_list = fn_get_unicode_value(129488,0,symbol_list)

        if "fant" in symbols: # fantasy
            do_job = True
            symbol_list = fn_get_unicode_value(128120,128128,symbol_list)

        if "figures" in symbols or "people" in symbols:# people
            do_job = True
            symbol_list = fn_get_unicode_value(129328,129329,symbol_list)
            symbol_list = fn_get_unicode_value(129332,129335,symbol_list)
            symbol_list = fn_get_unicode_value(129489,129503,symbol_list)

        if "flag" in symbols:# flags
            do_job = True
            symbol_list = fn_get_unicode_value(127985,127988,symbol_list)

        if "flow" in symbols:# flowers
            do_job = True
            symbol_list = fn_get_unicode_value(127799,127804,symbol_list)

        if "food" in symbols or "eating" in symbols:# food and eating
            do_job = True
            symbol_list = fn_get_unicode_value(129360,129387,symbol_list)
            symbol_list = fn_get_unicode_value(127789,127791,symbol_list)
            symbol_list = fn_get_unicode_value(127798,0,symbol_list)
            symbol_list = fn_get_unicode_value(127807,0,symbol_list)
            symbol_list = fn_get_unicode_value(127828,127850,symbol_list)
            symbol_list = fn_get_unicode_value(127854,127859,symbol_list)
            symbol_list = fn_get_unicode_value(127860,127870,symbol_list)

        if "fru" in symbols: # fruit
            do_job = True
            symbol_list = fn_get_unicode_value(127813,0,symbol_list)
            symbol_list = fn_get_unicode_value(127815,127827,symbol_list)

        if "gend" in symbols:# gender
            do_job = True
            symbol_list = fn_get_unicode_value(9890,9895,symbol_list)

        if "hand" in symbols or "gest" in symbols:# hands
            do_job = True
            symbol_list = fn_get_unicode_value(128398,128419,symbol_list)
            symbol_list = fn_get_unicode_value(129304,129311,symbol_list)
            symbol_list = fn_get_unicode_value(129330,129331,symbol_list)

        if "heal" in symbols or "medi" in symbols:# health medicine
            do_job = True
            symbol_list = fn_get_unicode_value(128137,128138,symbol_list)

        if "media" in symbols:
            do_job = True
            symbol_list = fn_get_unicode_value(128231,128280,symbol_list)
            symbol_list = fn_get_unicode_value(128360,128362,symbol_list)

        if "misc" in symbols:
            do_job = True
            symbol_list = fn_get_unicode_value(8488,8527,symbol_list)

        if "money" in symbols or "curr" in symbols or "fina" in symbols:# money currency finance
            do_job = True
            symbol_list = fn_get_unicode_value(8352,8383,symbol_list)
            symbol_list = fn_get_unicode_value(128176,128185,symbol_list)

        if "music" in symbols:
            do_job = True
            symbol_list = fn_get_unicode_value(9833,9839,symbol_list)
            symbol_list = fn_get_unicode_value(119040,119078,symbol_list)
            symbol_list = fn_get_unicode_value(119081,119106,symbol_list)
            symbol_list = fn_get_unicode_value(119130,119140,symbol_list)

        if "number" in symbols:
            do_job = True
            symbol_list = fn_get_unicode_value(8528,8587,symbol_list)

        if "pen" in symbols: #pen pencil
            do_job = True
            symbol_list = fn_get_unicode_value(9998,10002,symbol_list)

        if "peo" in symbols: # people
            do_job = True
            symbol_list = fn_get_unicode_value(128100,128120,symbol_list)
            symbol_list = fn_get_unicode_value(128129,128131,symbol_list)

        if "place" in symbols:# places
            do_job = True
            symbol_list = fn_get_unicode_value(127956,127966,symbol_list)

        if "point" in symbols:# pointers
            do_job = True
            symbol_list = fn_get_unicode_value(9754,9759,symbol_list)

        if "rect" in symbols:# Geometric Shapes
            do_job = True
            symbol_list = fn_get_unicode_value(9644,9647,symbol_list) # Geometric Shapes

        if "recy" in symbols:# recycling symbols
            do_job = True
            symbol_list = fn_get_unicode_value(9842,9854,symbol_list)

        if "rhom" in symbols:# Geometric Shapes
            do_job = True
            symbol_list = fn_get_unicode_value(9670,9672,symbol_list) # Geometric Shapes
            symbol_list = fn_get_unicode_value(9674,0,symbol_list) # Geometric Shapes
            symbol_list = fn_get_unicode_value(128919,128928,symbol_list) # Geometric Shapes Extended

        if "romanc" in symbols or "wedd" in symbols:# romance wedding
            do_job = True
            symbol_list = fn_get_unicode_value(128139,128160,symbol_list)

        if "relig" in symbols or "god" in symbols:# religion
            do_job = True
            symbol_list = fn_get_unicode_value(9765,9775,symbol_list)
            symbol_list = fn_get_unicode_value(9784,0,symbol_list)
            symbol_list = fn_get_unicode_value(128325,128334,symbol_list)

        if "runic" in symbols:
            do_job = True
            symbol_list = fn_get_unicode_value(5792,5872,symbol_list)

        if "scis" in symbols:# scissors
            do_job = True
            symbol_list = fn_get_unicode_value(9984,9988,symbol_list)

        if "shape" in symbols:
            do_job = True
            symbol_list = fn_get_unicode_value(9632,9727,symbol_list)
            symbol_list = fn_get_unicode_value(11039,11055,symbol_list)

        if "sign" in symbols:
            do_job = True
            symbol_list = fn_get_unicode_value(9760,9764,symbol_list)
            symbol_list = fn_get_unicode_value(9773,9774,symbol_list)
            symbol_list = fn_get_unicode_value(9874,9884,symbol_list)
            symbol_list = fn_get_unicode_value(9888,0,symbol_list)
            symbol_list = fn_get_unicode_value(9935,9939,symbol_list)

        if "space" in symbols:
            do_job = True
            symbol_list = fn_get_unicode_value(9788,9799,symbol_list)
            symbol_list = fn_get_unicode_value(9732,0,symbol_list)

        if "sport" in symbols or "activity" in symbols:  #129351
            do_job = True
            symbol_list = fn_get_unicode_value(9917,9918,symbol_list)
            symbol_list = fn_get_unicode_value(9975,9977,symbol_list)
            symbol_list = fn_get_unicode_value(127933,127955,symbol_list)
            symbol_list = fn_get_unicode_value(127992,127993,symbol_list)
            symbol_list = fn_get_unicode_value(129336,129342,symbol_list)
            symbol_list = fn_get_unicode_value(129349,0,symbol_list)
            symbol_list = fn_get_unicode_value(129351,129356,symbol_list)

        if "squar" in symbols:# Geometric Shapes
            do_job = True
            symbol_list = fn_get_unicode_value(9632,9643,symbol_list) # Geometric Shapes
            symbol_list = fn_get_unicode_value(9703,9707,symbol_list) # Geometric Shapes
            symbol_list = fn_get_unicode_value(9712,9715,symbol_list) # Geometric Shapes
            symbol_list = fn_get_unicode_value(9723,9726,symbol_list) # Geometric Shapes
            symbol_list = fn_get_unicode_value(128908,128918,symbol_list) # Geometric Shapes Extended

        if "star" in symbols:
            do_job = True
            symbol_list = fn_get_unicode_value(9956,9959,symbol_list)
            symbol_list = fn_get_unicode_value(10017,0,symbol_list) # Dingbats
            symbol_list = fn_get_unicode_value(10022,10032,symbol_list) # Dingbats
            symbol_list = fn_get_unicode_value(10036,10041,symbol_list) # Dingbats
            symbol_list = fn_get_unicode_value(128960,128980,symbol_list) # Geometric Shapes Extended

        if "stat" in symbols: # stationery
            do_job = True
            symbol_list = fn_get_unicode_value(128391,128397,symbol_list)

        if "swe" in symbols or "cand" in symbols or "conf" in symbols: # sweets candy confectionary
            do_job = True
            symbol_list = fn_get_unicode_value(127851,127853,symbol_list)
            symbol_list = fn_get_unicode_value(127871,0,symbol_list)

        if "tarot" in symbols:
            do_job = True
            symbol_list = fn_get_unicode_value(127200,127221,symbol_list)

        if "tech" in symbols:# technology
            do_job = True
            symbol_list = fn_get_unicode_value(9167,0,symbol_list)
            symbol_list = fn_get_unicode_value(9193,9199,symbol_list)
            symbol_list = fn_get_unicode_value(9204,9214,symbol_list)
            symbol_list = fn_get_unicode_value(128421,128443,symbol_list)

        if "tick" in symbols or "check" in symbols:
            do_job = True
            symbol_list = fn_get_unicode_value(10003,10004,symbol_list)

        if "transp" in symbols or "map" in symbols:# transport/map symbols
            do_job = True
            symbol_list = fn_get_unicode_value(128640,128722,symbol_list)
            symbol_list = fn_get_unicode_value(128725,0,symbol_list)
            symbol_list = fn_get_unicode_value(128736,128748,symbol_list)
            symbol_list = fn_get_unicode_value(128752,128762,symbol_list)

        if "tree" in symbols:
            do_job = True
            symbol_list = fn_get_unicode_value(127792,127796,symbol_list)
            symbol_list = fn_get_unicode_value(127809,127811,symbol_list)

        if "triang" in symbols:# Geometric Shapes
            do_job = True
            symbol_list = fn_get_unicode_value(9650,9669,symbol_list) # Geometric Shapes
            symbol_list = fn_get_unicode_value(9698,9701,symbol_list) # Geometric Shapes
            symbol_list = fn_get_unicode_value(9708,9710,symbol_list) # Geometric Shapes
            symbol_list = fn_get_unicode_value(9720,9722,symbol_list) # Geometric Shapes
            symbol_list = fn_get_unicode_value(9727,0,symbol_list) # Geometric Shapes
            symbol_list = fn_get_unicode_value(128896,128899,symbol_list) # Geometric Shapes Extended

        if "veg" in symbols:# vegetables
            do_job = True
            symbol_list = fn_get_unicode_value(127812,127814,symbol_list)

        if "vote" in symbols or "ballot" in symbols:
            do_job = True
            symbol_list = fn_get_unicode_value(9745,9746,symbol_list)
            symbol_list = fn_get_unicode_value(10007,10008,symbol_list)

        if "weather" in symbols:
            do_job = True
            symbol_list = fn_get_unicode_value(9728,9737,symbol_list)
            symbol_list = fn_get_unicode_value(9924,9928,symbol_list)
            symbol_list = fn_get_unicode_value(127744,127755,symbol_list)
            symbol_list = fn_get_unicode_value(127777,127788,symbol_list)
			
        if "Greek_language" in symbols:
            do_job = True
            symbol_list = fn_get_unicode_value(913,929,symbol_list)
            symbol_list = fn_get_unicode_value(931,969,symbol_list)			
			
	

        #### end of unicode character data
#################################### Rel,5 ############################################

	if display_keywords==1 or create_new==1:		
		image = pdb.gimp_image_new(font_size, font_size, 0)
		display = pdb.gimp_display_new(image)
		layer = pdb.gimp_layer_new(image, font_size, font_size, RGBA_IMAGE, "background",100, 0)
		pdb.gimp_image_insert_layer(image, layer, None, 0)

#######################################################################################
	if len(symbol_list)>1:

		loop = 1
		while loop < len(symbol_list): # loop through unicode list to find the max/min positions of characters in all layers
			#text_layer = pdb.gimp_text_fontname(image, None,0, 0, eval(symbol_list[loop]), 0, 1, font_size, 1, font_name)
			pdb.gimp_context_set_foreground((0,0,0))
			text_layer = pdb.gimp_text_fontname(image, None,0, 0, eval(symbol_list[loop]), 0, 1, font_size, 1, 1)			
			pdb.gimp_image_select_item(image, 2, text_layer) # character alpha to selection
			success,x0,x1,y0,y1, s_width, s_height = fn_selection_info(image) # get alpha selection info
			pdb.gimp_selection_none(image)

			if display_keywords==False:

				w = pdb.gimp_image_width(image)
				h = pdb.gimp_image_height(image)
				wh = pdb.gimp_drawable_width(text_layer)
				hh = pdb.gimp_drawable_height(text_layer)		
				centerx=w/2
				centery=h/2
				adw=wh/2
				adh=hh/2
				sw=centerx - adw
				sh=centery - adh		
				pdb.gimp_layer_set_offsets(text_layer, sw, sh)
	
			if success:
				if path==1:
					temp_vector_layer=pdb.gimp_vectors_new_from_text_layer(image, text_layer)
					pdb.gimp_image_insert_vectors(image, temp_vector_layer, None, -1)
					#pdb.gimp_item_transform_translate(temp_vector_layer,sw, sh)
			loop += 1
			
		if reverse:
			pdb.script_fu_reverse_layers(image, text_layer)
			pdb.gimp_image_lower_item_to_bottom(image, layer)
			
		pdb.gimp_image_resize_to_layers(image)
		
	if display_keywords:
		#if symbols=="Greek_language":
			#pdb.script_fu_reverse_layers(image, text_layer)
			
		w = pdb.gimp_image_width(image)
		h = pdb.gimp_image_height(image)
		
		for layer in image.layers:
		
			wh = pdb.gimp_drawable_width(layer)
			hh = pdb.gimp_drawable_height(layer)		
			centerx=w/2
			centery=h/2
			adw=wh/2
			adh=hh/2
			sw=centerx - adw
			sh=centery - adh		
			pdb.gimp_layer_set_offsets(layer, sw, sh)
		
			pdb.gimp_image_set_active_layer(image,layer)
			pdb.gimp_layer_resize_to_image_size(layer)

# add border
			wh = pdb.gimp_drawable_width(layer)
			hh = pdb.gimp_drawable_height(layer)			
			selection = pdb.gimp_item_is_selection(layer)
			pdb.gimp_selection_invert(image)
			pdb.gimp_image_select_rectangle(image, 2, 3, 3, wh-6, hh-6)
			pdb.gimp_selection_invert(image)
			pdb.gimp_image_select_rectangle(image, 3, 1, 1, wh-2, hh-2)
			pdb.gimp_context_set_foreground((0,0,0))
			pdb.gimp_edit_fill(layer, 0)
			pdb.gimp_selection_none(image)				
			
# create tiles
		pdb.gimp_image_remove_layer(image, layer)
		wrkimg=gimp.image_list()[0]
		wrklay=wrkimg.layers[-1]
		man = len(image.layers)  
		many = len(image.layers)/iter  
		ile = math.ceil(many)   
		if man != ile*iter:

			dup = (ile)*iter - man			# dup = (ile+1)*iter - man
			for item in range(0,int(dup)):
				
				copy=pdb.gimp_layer_copy(wrklay,1)
				new = pdb.gimp_image_add_layer(image,copy,-1)
				pdb.gimp_image_lower_item_to_bottom(image, copy)
				pdb.gimp_edit_clear(copy)
	
		#pdb.python_fu_ofn_tiles_to_layer(image, specs, geometry, rcOrder, stackOrder, flatten, resize)
		#pdb.python_fu_ofn_tiles_to_layer(image, 1, iter, 0, 0, 0, 1)
		tilesToLayerUnicode(image, 1, iter, 0, 0, 0, 1)
		pdb.gimp_context_set_foreground((h_color))
      # text_layer = pdb.gimp_text_fontname(image, drawable, x-left of the text bounding box, y-top of the text bounding box, text, border, antialias, size, size_type, fontname)		
		text_layer = pdb.gimp_text_fontname(image, None, 0, -font_size, ' UNICODE: ' + symbols, 5, 1, h_font_size, PIXELS, h_font)
		pdb.gimp_image_resize_to_layers(image)

		rkimg=gimp.image_list()[0]
		rklay=rkimg.layers[-1]
		pdb.gimp_layer_resize_to_image_size(rklay)

		wi = pdb.gimp_image_width(image)
		hi = pdb.gimp_image_height(image)
		margin =20
		new_width = wi + 2*margin
		new_height = hi + 2*margin
		
		wh = pdb.gimp_drawable_width(rklay)
		hh = pdb.gimp_drawable_height(rklay)
		centerx=wi/2
		centery=hi/2
		adw=wh/2
		adh=hh/2
		sw=centerx - adw
		sh=centery - adh		
		pdb.gimp_layer_set_offsets(rklay, sw, sh)
		pdb.gimp_image_resize(image,new_width,new_height,margin, margin)
		pdb.gimp_layer_resize_to_image_size(rklay)
		if man == ile*iter:
			layer = pdb.gimp_layer_new(image, new_width,new_height, RGBA_IMAGE, "background",100, 0)
			pdb.gimp_image_insert_layer(image, layer, None, 0)
			pdb.gimp_edit_fill(layer, 2)
			pdb.gimp_image_lower_item_to_bottom(image, layer)
		if man != ile*iter:			
			pdb.gimp_edit_fill(rklay, 2)
			pdb.gimp_context_set_foreground((0,0,0))

######################################################################################################################################		
def fn_get_unicode_value(uni_input1,uni_input2, symbol_list):
    if uni_input2>0:
        for uni_val in range(uni_input1, uni_input2+1):
            hex_string = fn_unicode_hex(uni_val)
            symbol_list.append(hex_string)
    else:
        hex_string = fn_unicode_hex(uni_input1)
        symbol_list.append(hex_string)
    return symbol_list

######################################################################################################################################

def fn_unicode_hex(dec_number):
    # convert a number into hex value string to use with unicode escape sequence
    hex_lead = "U" + "\'" + "\\" + "U"
    hex_tail = "\'"
    hex_middle = str(hex(dec_number)).upper()
    hex_middle = hex_middle[2:len(hex_middle)]
    zero_string = ""
    if len(hex_middle)<8:
        zero_string = "0" * (8 - len(hex_middle))
    hex_name = hex_lead + zero_string + hex_middle + hex_tail
    return hex_name

######################################################################################################################################

    #### Useful Functions
# -------------------------------------------------------------------------------------

def fn_selection_info(im):
    # returns selection info
    selection,x1,y1,x2,y2 = pdb.gimp_selection_bounds(im)
    s_width, s_height = x2 - x1, y2 - y1

    return selection, x1, x2, y1, y2, s_width, s_height

######################################################################################################################################

def proc_messagebox(message):
    old_handler = pdb.gimp_message_get_handler()
    pdb.gimp_message_set_handler(2)
    pdb.gimp_message(str(message))
    pdb.gimp_message_set_handler(old_handler)

######################################################################################################################################
# 
register(
	"python_fu_character_groups_UNICODE",
	"UNICODE Character Groups",
	"80 Character Groups UNICODE by Skinnyhouse [e.g. as a source for BabelMap (text) to GIMP]",
	"Skinnyhouse & MareroQ",
	"GPL",
	"2019",
	"Character groups UNICODE...",  # 80  character groups UNICODE...
	"*",
	[
	(PF_IMAGE, "image", "Input image", None), #current image
	(PF_DRAWABLE, "layer", "Input layer", None), #current layer 
	(PF_SPINNER, "pf_fontsize", "Font size:", 200, (10, 1000, 1)),
	(PF_OPTION, "symbols", "Name group: ", 0, ["Activity (Sport)", "Animal", "Arrow","Asterisk","Astrology","Ballot","Beauty","Body","Braille","Bubble","Building","Bullet",
	"Candy","Celebration","Cereal","Chess","Circle","Clock","Clothes","Clubs","Communication","Cross","Currency",
	"Diamonds","Dice","Dingbat","Domino","Eating","Emoticon","Entertainment",
	"Face","Fantasy","Figures","Finance","Flag","Flower","Food","Fruit",
	"Gender","Gestures","Hands","Health","Hearts","Map","Media","Medicine","Misc","Money","Music","Number",
	"Pen","People","Places","Pointer","Rectangle","Recycling","Rhombus","Religion","Romance","Runic",
	"Scissors","Shape","Sign","Space","Spades","Square","Star","Stationery","Sweets","Suit",
	"Tarot","Technology","Tick","Transport","Tree","Triangle","Vegetable","Vote","Weather","Greek language"
	]),
	(PF_BOOL,    "create_new", "Create new image:", True),
	(PF_TOGGLE,  "patch", "Create path:", 1),	
	(PF_TOGGLE,  "pf_list_veryspecials", "Display as tiles (new image):", 0),		# display_keywords,
	(PF_TOGGLE,  "reverse", "Reverse Layer Order:", 0),
	(PF_SPINNER, "iter", "Number of Tile Columns:", 10, (2, 30, 1)),
	(PF_FONT,    "h_font", "Heading - Font Name:", "Arial"),	
	(PF_SPINNER, "h_font_size", "Heading - Font Size:", 100, (10, 500, 1)),
	(PF_COLOR,   "h_color", "Heading - Font Color:", (0, 0, 0)),	
	],
	[],
	character_groups,
	menu="<Image>/Filters/Path/"u'\u25cf'" Unicode to shape/",)

main()
